"""
Script parameter handling utility.
"""

import configparser

from typing import List, Any


class GenPars(object):
    """
    Main class for genpars.
    """

    def __init__(self, parfile: str):

        self.parfile: str = parfile
        self.parfile_dict: dict = {}

    def parfile_read(self, parfilename: str = None) -> None:
        """
        Read a parameter file and create a dictionary for saving
        parameters between sessions.

        :param str parfilename: Name of parameter file
        :return: None
        """

        self.parfile_dict = {"server": {"wd": None}}  # special case for missing par file

        if parfilename is None:
            parfilename = self.parfile
            if parfilename is None:
                raise FileNotFoundError("Parameter file is not defined")

        cp = configparser.ConfigParser()
        cp.read(parfilename)

        sections = cp.sections()

        # sectionname & value case sensitive, name is not
        for sectionname in sections:

            self.parfile_dict[sectionname] = {}

            for name, value in cp.items(sectionname):
                name = name.lower()
                self.parfile_dict[sectionname][name] = value

        return

    def parfile_write(self, parfilename: str = None) -> None:
        """
        Update a parameter file with current values.

        :param str parfilename: Name of parameter file
        :return: None
        """

        if parfilename is None or parfilename == "None":
            parfilename = self.parfile

        if parfilename is None:
            raise FileNotFoundError("Parameter file is not defined")
            return

        config = configparser.ConfigParser()

        for sectionname in self.parfile_dict:
            for par in self.parfile_dict[sectionname]:
                if self.parfile_dict[sectionname][par] is None:
                    self.parfile_dict[sectionname][par] = "None"
            config[sectionname] = self.parfile_dict[sectionname]

        # write parfile
        with open(parfilename, "w") as configfile:
            config.write(configfile)

        return

    def get_parfile_par(
        self,
        caller: str,
        attribute: any,
        value: any = "default",
        prompt_string: str = "",
        default: any = None,
    ):
        """
        Return a parameter from the parfile_dict database, or prompt as needed.
        The new value is saved in the database.

        :param str caller: Name of caller, used as dictionary name
        :param str attribute: Name of attribute, used as dictionary key
        :param str value: "default" or "prompt" or a value
        :param str prompt: Prompt message
        :param str default: Default value to be used
        :return str:  parameter
        """

        # check if dictionary for caller does not exist
        if not self.parfile_dict.get(caller):
            self.parfile_dict[caller] = {}

        # check if a value exists in dictionary
        if self.parfile_dict[caller].get(attribute):
            default = self.parfile_dict[caller].get(attribute)

        if value == "prompt":
            if prompt_string == "":
                prompt_string = f"Enter value for {attribute}"
            value = self.prompt(prompt_string, default)
            _, value = self.get_datatype(value)
        elif value == "default":
            value = default
        else:
            pass  # value passsed is used

        self.parfile_dict[caller][attribute] = value

        return value

    def set_parfile_par(self, caller, attribute, value):
        """
        Set a parameter from the parfile_dict database.

        :param str caller: Name of caller, used as dictionary name
        :param str attribute: Name of attribute, used as dictionary key
        :param str value: "default" or "prompt" or a value
        :return:  None
        """

        # check if dictionary for caller does not exist
        if not self.parfile_dict.get(caller):
            self.parfile_dict[caller] = {}

        # get value
        self.parfile_dict[caller][attribute] = value

        return

    def get_datatype(self, value: any) -> List:
        """
        Determine the data type for an object and set the type if possible. A string such as "1.23"
        will result in a type "float" and "2" will result in type "int".

        :param value: object to be typed
        :return [type, value]: list of data type as a code and object with that type
        """

        if type(value) is str:

            # string integer
            if value.isdigit():
                attributetype = "int"
                value = int(value)
                return [attributetype, value]
            else:
                try:
                    value = float(value)
                    attributetype = "float"
                    return [attributetype, value]
                except ValueError:
                    pass

            attributetype = "str"

        elif type(value) is int:
            attributetype = "int"
            value = int(value)

        elif type(value) is float:
            attributetype = "float"
            value = float(value)

        # more work here
        else:
            attributetype = "str"

        return [attributetype, value]

    def prompt(self, message="Enter a string", default=""):
        """
        Prints a message and waits for user input.

        :param str PromptMessage: String to be printed
        :param str Default:  String to be returned if no value is entered
        :return str: String entered or Default
        """

        default = str(default)
        try:
            if default != "":
                in1 = input(message + " [" + default + "]: ")
            else:
                in1 = input(message + ": ")
        except KeyboardInterrupt:
            return ""

        if in1 == "":
            return default
        else:
            return in1
